@@ -1,7 +1,6 @@
 #' @title Create a controller group.
 #' @export
-#' @keywords internal
-#' @family controllers
+#' @family user
 #' @description Create an `R6` object to submit tasks and launch workers
 #'   through multiple `crew` controllers.
 #' @param ... `R6` controller objects or lists of `R6` controller objects.
@@ -9,25 +8,21 @@
 #'   or another list.
 #' @examples
 #' if (identical(Sys.getenv("CREW_EXAMPLES"), "true")) {
-#' crew_session_start()
 #' persistent <- crew_controller_local(name = "persistent")
 #' transient <- crew_controller_local(
 #'   name = "transient",
-#'   max_tasks = 1L
+#'   tasks_max = 1L
 #' )
 #' group <- crew_controller_group(persistent, transient)
-#' group$connect()
+#' group$start()
 #' group$push(name = "task", command = sqrt(4), controller = "transient")
 #' group$wait()
 #' group$pop()
-#' group$controllers[["persistent"]]$launcher$running() # 0
-#' group$controllers[["transient"]]$launcher$running() # 0
 #' group$terminate()
-#' crew_session_terminate()
 #' }
 crew_controller_group <- function(...) {
   controllers <- unlist(list(...), recursive = TRUE)
-  names(controllers) <- map_chr(controllers, ~.x$router$name)
+  names(controllers) <- map_chr(controllers, ~.x$client$name)
   out <- crew_class_controller_group$new(controllers = controllers)
   out$validate()
   out
@@ -35,26 +30,22 @@ crew_controller_group <- function(...) {
 
 #' @title Controller group class
 #' @export
-#' @family controllers
+#' @family class
 #' @description `R6` class for controller groups.
 #' @details See [crew_controller_group()].
 #' @examples
 #' if (identical(Sys.getenv("CREW_EXAMPLES"), "true")) {
-#' crew_session_start()
 #' persistent <- crew_controller_local(name = "persistent")
 #' transient <- crew_controller_local(
 #'   name = "transient",
-#'   max_tasks = 1L
+#'   tasks_max = 1L
 #' )
 #' group <- crew_controller_group(persistent, transient)
-#' group$connect()
+#' group$start()
 #' group$push(name = "task", command = sqrt(4), controller = "transient")
 #' group$wait()
 #' group$pop()
-#' group$controllers[["persistent"]]$launcher$running() # 0
-#' group$controllers[["transient"]]$launcher$running() # 0
 #' group$terminate()
-#' crew_session_terminate()
 #' }
 crew_class_controller_group <- R6::R6Class(
   classname = "crew_class_controller_group",
@@ -82,6 +73,15 @@ crew_class_controller_group <- R6::R6Class(
         )
       )
       self$controllers[names]
+    },
+    select_single_controller = function(name) {
+      names <- names(self$controllers)
+      name <- name %|||% utils::head(names, n = 1L)
+      crew_assert(
+        name %in% names,
+        message = sprintf("controller not found: %s", name)
+      )
+      self$controllers[[name]]
     }
   ),
   public = list(
@@ -92,21 +92,17 @@ crew_class_controller_group <- R6::R6Class(
     #' @param controllers List of `R6` controller objects.
     #' @examples
     #' if (identical(Sys.getenv("CREW_EXAMPLES"), "true")) {
-    #' crew_session_start()
     #' persistent <- crew_controller_local(name = "persistent")
     #' transient <- crew_controller_local(
     #'   name = "transient",
-    #'   max_tasks = 1L
+    #'   tasks_max = 1L
     #' )
     #' group <- crew_controller_group(persistent, transient)
-    #' group$connect()
+    #' group$start()
     #' group$push(name = "task", command = sqrt(4), controller = "transient")
     #' group$wait()
     #' group$pop()
-    #' group$controllers[["persistent"]]$launcher$running() # 0
-    #' group$controllers[["transient"]]$launcher$running() # 0
     #' group$terminate()
-    #' crew_session_terminate()
     #' }
     initialize = function(
       controllers = NULL
@@ -114,14 +110,14 @@ crew_class_controller_group <- R6::R6Class(
       self$controllers <- controllers
       invisible()
     },
-    #' @description Validate the router.
+    #' @description Validate the client.
     #' @return `NULL` (invisibly).
     validate = function() {
       crew_assert(
         map_lgl(self$controllers, ~inherits(.x, "crew_class_controller")),
         message = "All objects in a controller group must be controllers."
       )
-      out <- unname(map_chr(self$controllers, ~.x$router$name))
+      out <- unname(map_chr(self$controllers, ~.x$client$name))
       exp <- names(self$controllers)
       crew_assert(identical(out, exp), message = "bad controller names")
       invisible()
@@ -137,6 +133,30 @@ crew_class_controller_group <- R6::R6Class(
       control <- private$select_controllers(controllers)
       all(map_lgl(control, ~.x$empty()))
     },
+    #' @description Check if a controller is saturated.
+    #' @details A controller is saturated if the number of unresolved tasks
+    #'   is greater than or equal to the maximum number of workers.
+    #'   In other words, in a saturated controller, every available worker
+    #'   has a task.
+    #'   You can still push tasks to a saturated controller, but
+    #'   tools that use `crew` such as `targets` may choose not to.
+    #' @return `TRUE` if all the selected controllers are saturated,
+    #'   `FALSE` otherwise.
+    #' @param collect Logical of length 1, whether to collect the results
+    #'   of any newly resolved tasks before determining saturation.
+    #' @param throttle Logical of length 1, whether to delay task collection
+    #'   until the next request at least `self$client$seconds_interval`
+    #'   seconds from the original request.
+    #'   The idea is similar to `shiny::throttle()` except that `crew` does not
+    #'   accumulate a backlog of requests. The technique improves robustness
+    #'   and efficiency.
+    #' @param controller Character vector of length 1 with the controller name.
+    #'   Set to `NULL` to select the default controller that `push()`
+    #'   would choose.
+    saturated = function(collect = TRUE, throttle = TRUE, controller = NULL) {
+      control <- private$select_single_controller(name = controller)
+      control$saturated(collect = collect, throttle = throttle)
+    },
     #' @description Start one or more controllers.
     #' @return `NULL` (invisibly).
     #' @param controllers Character vector of controller names.
@@ -158,11 +178,17 @@ crew_class_controller_group <- R6::R6Class(
     #'   in one or more controller objects.
     #' @details See the `scale()` method in individual controller classes.
     #' @return `NULL` (invisibly).
+    #' @param throttle Logical of length 1, whether to delay auto-scaling
+    #'   until the next auto-scaling request at least
+    #'  `self$client$seconds_interval` seconds from the original request.
+    #'   The idea is similar to `shiny::throttle()` except that `crew` does not
+    #'   accumulate a backlog of requests. The technique improves robustness
+    #'   and efficiency.
     #' @param controllers Character vector of controller names.
     #'   Set to `NULL` to select all controllers.
-    scale = function(controllers = NULL) {
+    scale = function(throttle = FALSE, controllers = NULL) {
       control <- private$select_controllers(controllers)
-      walk(control, ~.x$scale())
+      walk(control, ~.x$scale(throttle = throttle))
     },
     #' @description Push a task to the head of the task list.
     #' @return `NULL` (invisibly).
@@ -191,15 +217,26 @@ crew_class_controller_group <- R6::R6Class(
     #' @param seconds_timeout Optional task timeout passed to the `.timeout`
     #'   argument of `mirai::mirai()` (after converting to milliseconds).
     #' @param scale Logical, whether to automatically scale workers to meet
-    #'   demand. If `TRUE`, then `collect()` runs first
+    #'   demand.
+    #'   If `TRUE`, then `collect()` runs first
     #'   so demand can be properly assessed before scaling and the number
     #'   of workers is not too high.
+    #' @param throttle If `scale` is `TRUE`, whether to defer auto-scaling
+    #'   until the next auto-scaling request at least
+    #'   `self$client$seconds_interval` seconds from the original request.
+    #'   The idea is similar to `shiny::throttle()` except that `crew` does not
+    #'   accumulate a backlog of requests. The technique improves robustness
+    #'   and efficiency.
     #' @param name Optional name of the task. Replaced with a random name
     #'   if `NULL` or in conflict with an existing name in the task list.
     #' @param controller Character of length 1,
     #'   name of the controller to submit the task.
     #'   If `NULL`, the controller defaults to the
     #'   first controller in the list.
+    #' @param save_command Logical of length 1. If `TRUE`, the controller
+    #'   deparses the command and returns it with the output on `pop()`.
+    #'   If `FALSE` (default), the controller skips this step to
+    #'   increase speed.
     push = function(
       command,
       data = list(),
@@ -210,43 +247,48 @@ crew_class_controller_group <- R6::R6Class(
       library = NULL,
       seconds_timeout = NULL,
       scale = TRUE,
+      throttle = TRUE,
       name = NULL,
+      save_command = FALSE,
       controller = NULL
     ) {
-      controller <- controller %|||%
-        utils::head(names(self$controllers), n = 1L)
-      crew_assert(
-        length(controller) == 1L,
-        message = "controller argument of push() must have length 1."
-      )
-      crew_assert(
-        controller %in% names(self$controllers),
-        message = sprintf("controller \"%s\" not found", controller)
-      )
-      if (substitute) command <- substitute(command)
-      args <- list(
+      if (substitute) {
+        command <- substitute(command)
+      }
+      control <- .subset2(
+        private,
+        "select_single_controller"
+      )(name = controller)
+      .subset2(control, "push")(
         command = command,
         data = data,
         globals = globals,
-        substitute = TRUE,
+        substitute = FALSE,
         seed = seed,
         packages = packages,
         library = library,
         seconds_timeout = seconds_timeout,
+        scale = scale,
+        throttle = throttle,
         name = name,
-        scale = scale
+        save_command = save_command
       )
-      do.call(what = self$controllers[[controller]]$push, args = args)
     },
     #' @description Check for done tasks and move the results to
     #'   the results list.
     #' @return `NULL` (invisibly). Removes elements from the `queue`
     #'   list as applicable and moves them to the `results` list.
+    #' @param throttle whether to defer task collection
+    #'   until the next task collection request at least
+    #'   `self$client$seconds_interval` seconds from the original request.
+    #'   The idea is similar to `shiny::throttle()` except that `crew` does not
+    #'   accumulate a backlog of requests. The technique improves robustness
+    #'   and efficiency.
     #' @param controllers Character vector of controller names.
     #'   Set to `NULL` to select all controllers.
-    collect = function(controllers = NULL) {
+    collect = function(throttle = FALSE, controllers = NULL) {
       control <- private$select_controllers(controllers)
-      walk(control, ~.x$collect())
+      walk(control, ~.x$collect(throttle = throttle))
     },
     #' @description Pop a completed task from the results data frame.
     #' @return If there is a completed task available to collect, the return
@@ -254,17 +296,37 @@ crew_class_controller_group <- R6::R6Class(
     #'   Otherwise, if there are no results available to collect,
     #'   the return value is `NULL`.
     #' @param scale Logical, whether to automatically scale workers to meet
-    #'   demand. If `TRUE`, then `collect()` runs first
+    #'   demand.
+    #'   If `TRUE`, then `collect()` runs first
     #'   so demand can be properly assessed before scaling and the number
     #'   of workers is not too high. Scaling up on `pop()` may be important
     #'   for transient or nearly transient workers that tend to drop off
     #'   quickly after doing little work.
+    #' @param collect Logical of length 1. If `scale` is `FALSE`,
+    #'   whether to call `collect()`
+    #'   to pick up the results of completed tasks. This task collection
+    #'   step always happens (with throttling) when `scale` is `TRUE`.
+    #' @param throttle If `scale` is `TRUE`, whether to defer auto-scaling
+    #'   until the next auto-scaling request at least
+    #'   `self$client$seconds_interval` seconds from the original request.
+    #'   The idea is similar to `shiny::throttle()` except that `crew` does not
+    #'   accumulate a backlog of requests. The technique improves robustness
+    #'   and efficiency.
     #' @param controllers Character vector of controller names.
     #'   Set to `NULL` to select all controllers.
-    pop = function(scale = TRUE, controllers = NULL) {
+    pop = function(
+      scale = TRUE,
+      collect = TRUE,
+      throttle = TRUE,
+      controllers = NULL
+    ) {
       control <- private$select_controllers(controllers)
       for (controller in control) {
-        out <- controller$pop(scale = scale)
+        out <- controller$pop(
+          scale = scale,
+          collect = collect,
+          throttle = throttle
+        )
         if (!is.null(out)) {
           return(out)
         }
@@ -286,12 +348,26 @@ crew_class_controller_group <- R6::R6Class(
     #'   polling intervals while checking for results.
     #' @param seconds_timeout Timeout length in seconds waiting for
     #'   results to become available.
+    #' @param scale Logical of length 1, whether to call `scale_later()`
+    #'   on each selected controller to schedule auto-scaling.
+    #'   By design, auto-scaling might not actually happen on
+    #'   every iteration if `throttle` is `TRUE`.
+    #' @param throttle If `scale` is `TRUE`, whether to defer auto-scaling
+    #'   and task collection until the next request at least
+    #'   `self$client$seconds_interval` seconds from the original request.
+    #'   The idea is similar to `shiny::throttle()` except that `crew` does not
+    #'   accumulate a backlog of requests. The technique improves robustness
+    #'   and efficiency.
+    #'   Highly recommended to keep
+    #'   the default `throttle = TRUE` for `wait()`.
     #' @param controllers Character vector of controller names.
     #'   Set to `NULL` to select all controllers.
     wait = function(
       mode = "all",
-      seconds_interval = 0.001,
+      seconds_interval = 0.01,
       seconds_timeout = Inf,
+      scale = TRUE,
+      throttle = TRUE,
       controllers = NULL
     ) {
       mode <- as.character(mode)
@@ -302,19 +378,16 @@ crew_class_controller_group <- R6::R6Class(
           fun = ~{
             empty <- TRUE
             for (controller in control) {
-              controller$collect()
-              controller$scale()
-              empty_queue <- length(controller$queue) < 1L
-              empty_results <- length(controller$results) < 1L
-              done <- if_any(
-                identical(mode, "all"),
-                empty_queue && (!empty_results),
-                empty_queue || (!empty_results)
-              )
-              if (done) {
+              controller$schedule$collect(throttle = throttle)
+              if (scale) {
+                controller$scale(throttle = throttle)
+              }
+              done <- controller$schedule$collected_mode(mode = mode)
+              empty <- controller$schedule$empty()
+              if (done && !empty) {
                 return(TRUE)
               }
-              empty <- empty && empty_queue && empty_results
+              empty <- empty && controller$schedule$empty()
             }
             empty
           },
@@ -327,31 +400,21 @@ crew_class_controller_group <- R6::R6Class(
     },
     #' @description Summarize the workers of one or more controllers.
     #' @return A data frame of aggregated worker summary statistics
-    #'   of all the selected controllers. It has one row per worker
-    #'   websocket, and the rows are grouped by controller.
+    #'   of all the selected controllers. It has one row per worker,
+    #'   and the rows are grouped by controller.
     #'   See the documentation of the `summary()` method of the controller
     #'   class for specific information about the columns in the output.
-    #' @param columns Tidyselect expression to select a subset of columns.
-    #'   Examples include `columns = contains("worker")` and
-    #'   `columns = starts_with("tasks")`.
     #' @param controllers Character vector of controller names.
     #'   Set to `NULL` to select all controllers.
-    summary = function(
-      columns = tidyselect::everything(),
-      controllers = NULL
-    ) {
+    summary = function(controllers = NULL) {
       control <- private$select_controllers(controllers)
-      columns <- rlang::enquo(columns)
-      out <- map(
-        control,
-        ~do.call(what = .x$summary, args = list(columns = columns))
-      )
+      out <- map(control, ~.x$summary())
       if (all(map_lgl(out, is.null))) {
-        return(NULL)
+        return(NULL) # nocov
       }
       tibble::as_tibble(do.call(what = rbind, args = out))
     },
-    #' @description Terminate the workers and disconnect the router
+    #' @description Terminate the workers and disconnect the client
     #'   for one or more controllers.
     #' @return `NULL` (invisibly).
     #' @param controllers Character vector of controller names.
