@@ -6,8 +6,9 @@
 #' @details The randomness is not reproducible and cannot be set with
 #'   e.g. `set.seed()` in R.
 #' @return A random character string.
+#' @param n Length of the random vector to hash.
 #' @examples
 #' crew_random_name()
-crew_random_name <- function() {
-  nanonext::sha1(nanonext::random(n = 100L))
+crew_random_name <- function(n = 1L) {
+  nanonext::sha1(c(nanonext::random(n = n), as.numeric(Sys.time())))
 }
