@@ -6,12 +6,13 @@ controller <- crew_controller_local(
 controller$start()
 names <- character(0L)
 index <- 0L
-n_tasks <- 6000L
+n_tasks <- 60000L
 system.time(
   while (index < n_tasks || !(controller$empty())) {
     if (index < n_tasks) {
       index <- index + 1L
       cat("submit", index, "\n")
+      cat("submit", index, "\n")
       controller$push(
         name = as.character(index),
         command = TRUE
@@ -24,5 +25,11 @@ system.time(
     }
   }
 )
-testthat::expect_equal(length(controller$queue), 0L)
+testthat::expect_equal(sort(as.integer(names)), seq_len(n_tasks))
+controller$launcher$workers$launched <- FALSE
+controller$launcher$tally()
 controller$terminate()
+testthat::expect_equal(length(controller$schedule$pushed), 0L)
+testthat::expect_equal(length(controller$schedule$collected), 0L)
+testthat::expect_equal(sum(controller$launcher$workers$assigned), n_tasks)
+testthat::expect_equal(sum(controller$launcher$workers$complete), n_tasks)
